import { getNameOfDescriptor } from "../helpers/getNameOfDescriptor.js";
export class ServiceInstanceInfo {
    descriptor;
    get instance() {
        if (!this.initialized) {
            throw new Error(`Service "${getNameOfDescriptor(this.descriptor)}" is not initialized (circular dependency)`);
        }
        return this._instance;
    }
    initialized;
    disposed;
    dependents;
    _instance;
    constructor(descriptor) {
        this.descriptor = descriptor;
        this.initialized = false;
        this.disposed = false;
        this.dependents = new Set();
    }
    initialize(creator) {
        if (this.initialized) {
            return;
        }
        this._instance = this.descriptor.dry ? null : creator();
        this.initialized = true;
    }
    addDependent(dependent) {
        this.dependents.add(dependent);
    }
    disposeSync() {
        const instance = this.instance;
        const disposeMethod = instance?.[Symbol.dispose] ?? instance?.dispose;
        if (typeof disposeMethod === "function" &&
            !this.disposed &&
            this.initialized) {
            for (const dependent of this.dependents) {
                dependent.disposeSync();
            }
            Reflect.apply(disposeMethod, instance, []);
            this.disposed = true;
        }
    }
    async disposeAsync() {
        const instance = this.instance;
        if (!this.disposed && this.initialized) {
            const disposeMethod = instance?.[Symbol.asyncDispose] ?? instance?.dispose;
            if (typeof disposeMethod === "function") {
                this.disposed = true;
                try {
                    await Promise.all([...this.dependents].map((dependent) => dependent.disposeAsync()));
                    await Reflect.apply(disposeMethod, instance, []);
                }
                catch (e) {
                    this.disposed = false;
                    throw e;
                }
            }
            else {
                this.disposeSync();
            }
        }
    }
}
//# sourceMappingURL=ServiceInstanceInfo.js.map