import { ServiceInstanceInfo } from "./ServiceInstanceInfo.js";
import { isServiceProvider } from "../helpers/isServiceProvider.js";
export class ServiceInstancesStore {
    instances;
    constructor() {
        this.instances = new Map();
    }
    [Symbol.iterator]() {
        return this.instances.values();
    }
    hasInstanceOf(descriptor) {
        return this.instances.has(descriptor);
    }
    getInstanceInfo(descriptor) {
        return this.instances.get(descriptor);
    }
    addInstance(descriptor, requestedBy) {
        let instanceInfo = this.getInstanceInfo(descriptor);
        if (!instanceInfo) {
            instanceInfo = new ServiceInstanceInfo(descriptor);
            this.instances.set(descriptor, instanceInfo);
        }
        if (requestedBy) {
            const dependentInstanceInfo = this.getInstanceInfo(requestedBy);
            if (dependentInstanceInfo) {
                instanceInfo.addDependent(dependentInstanceInfo);
            }
        }
        return instanceInfo;
    }
    [Symbol.dispose]() {
        for (const instanceInfo of this) {
            if (!isServiceProvider(instanceInfo.descriptor.service)) {
                instanceInfo.disposeSync();
            }
        }
        this.instances.clear();
    }
    async [Symbol.asyncDispose]() {
        await Promise.all([...this.instances.values()].map(async (instanceInfo) => {
            if (!isServiceProvider(instanceInfo.descriptor.service)) {
                await instanceInfo.disposeAsync();
            }
        }));
        this.instances.clear();
    }
}
//# sourceMappingURL=ServiceInstancesStore.js.map