import { getNameOfServiceType } from "../helpers/getNameOfServiceType.js";
import { BaseServiceImplementationResolver, isServiceImplementationResolver, } from "./BaseServiceImplementationResolver.js";
import { Debug } from "../debug.js";
import { ValueServiceImplementationResolver } from "./ValueServiceImplementationResolver.js";
import { IServiceProvider } from "../di/IServiceProvider.js";
import { single } from "../service-type-resolvers/single.js";
export class FactoryServiceImplementationResolver extends BaseServiceImplementationResolver {
    implementation;
    get name() {
        return getNameOfServiceType(this.implementation);
    }
    constructor(implementation) {
        super();
        this.implementation = implementation;
    }
    *resolve(getService, descriptor, requestedBy, mode) {
        let implementation = this.implementation;
        if (isServiceImplementationResolver(implementation)) {
            implementation = (yield* implementation.resolve(getService, descriptor, requestedBy, mode));
        }
        if (typeof implementation === "function") {
            const serviceProvider = yield* getService(single(IServiceProvider), requestedBy, mode);
            return () => {
                try {
                    return implementation(serviceProvider);
                }
                catch (err) {
                    if (err instanceof TypeError &&
                        err.message.includes("cannot be invoked without 'new'")) {
                        throw new Error(Debug.errors.classNotDecorated(implementation.name), {
                            cause: err,
                        });
                    }
                    else {
                        throw err;
                    }
                }
            };
        }
        return yield* new ValueServiceImplementationResolver(implementation).resolve(getService, descriptor, requestedBy, mode);
    }
}
//# sourceMappingURL=FactoryServiceImplementationResolver.js.map