import { getNameOfServiceType } from "../helpers/getNameOfServiceType.js";
import { BaseServiceImplementationResolver, isServiceImplementationResolver, } from "./BaseServiceImplementationResolver.js";
import { ServiceRegistry } from "../di/ServiceRegistry.js";
import { FactoryServiceImplementationResolver } from "./FactoryServiceImplementationResolver.js";
export class RegistryServiceImplementationResolver extends BaseServiceImplementationResolver {
    implementation;
    get name() {
        return getNameOfServiceType(this.implementation);
    }
    constructor(implementation) {
        super();
        this.implementation = implementation;
    }
    *resolve(internalGetService, descriptor, requestedBy, mode) {
        let implementation = this.implementation;
        if (isServiceImplementationResolver(implementation)) {
            implementation = (yield* implementation.resolve(internalGetService, descriptor, requestedBy, mode));
        }
        const metadata = ServiceRegistry.get(implementation);
        if (metadata) {
            const instanceDependencies = [];
            requestedBy = new Set([...requestedBy, descriptor]);
            for (const dependency of metadata.dependencies) {
                instanceDependencies.push(yield* internalGetService(dependency, requestedBy, mode));
            }
            return () => new implementation(...instanceDependencies);
        }
        return yield* new FactoryServiceImplementationResolver(implementation).resolve(internalGetService, descriptor, requestedBy, mode);
    }
}
//# sourceMappingURL=RegistryServiceImplementationResolver.js.map