import { Debug } from "../debug.js";
import { getNameOfDescriptor } from "../helpers/getNameOfDescriptor.js";
import { getNameOfServiceType } from "../helpers/getNameOfServiceType.js";
import { resolveGeneratorAsync } from "../helpers/resolveGeneratorAsync.js";
import { resolveGeneratorSync } from "../helpers/resolveGeneratorSync.js";
import { BaseServiceTypeResolver, isServiceTypeResolver, } from "./BaseServiceTypeResolver.js";
export class OptionalServiceTypeResolver extends BaseServiceTypeResolver {
    constructor(next) {
        super(next);
    }
    *resolve(collection, instancesStore, resolveServiceImplementation, requestedBy, mode, descriptor) {
        let next = this.next;
        function validateConstructorRequest() {
            const lastRequestedBy = [...requestedBy].pop();
            if (lastRequestedBy &&
                instancesStore.getInstanceInfo(lastRequestedBy)?.initialized === false) {
                throw new Error(Debug.errors.optionalServiceAsDependency(getNameOfServiceType(next), getNameOfDescriptor(lastRequestedBy)));
            }
        }
        if (isServiceTypeResolver(next)) {
            return new OptionalResolver(function* resolver(mode) {
                validateConstructorRequest();
                return yield* next.resolve(collection, instancesStore, resolveServiceImplementation, requestedBy, mode, descriptor);
            });
        }
        return new OptionalResolver(function* resolver(mode) {
            validateConstructorRequest();
            return yield* resolveServiceImplementation(descriptor ?? collection.getDescriptor(next), requestedBy, mode);
        });
    }
}
const NOT_SET = Symbol();
class OptionalResolver {
    resolveInternal;
    value;
    loader;
    constructor(resolveInternal) {
        this.resolveInternal = resolveInternal;
        this.value = NOT_SET;
        this.loader = null;
    }
    resolve() {
        if (this.value === NOT_SET) {
            this.value = resolveGeneratorSync(this.resolveInternal("sync"));
        }
        return this.value;
    }
    async resolveAsync() {
        if (this.value === NOT_SET) {
            if (!this.loader) {
                this.loader = resolveGeneratorAsync(this.resolveInternal("async"))
                    .then((value) => {
                    this.value = value;
                    return value;
                })
                    .finally(() => {
                    this.loader = null;
                });
            }
            this.value = await this.loader;
        }
        return this.value;
    }
}
export function isOptionalServiceTypeResolver(value) {
    return value instanceof OptionalServiceTypeResolver;
}
//# sourceMappingURL=OptionalServiceTypeResolver.js.map